#!/usr/bin/env python
# -*- coding= utf-8 -*-

"""
Program to test polyline map-relative and view-relative placement.
Select what to show and experiment with placement parameters.

Usage: test_polyline_placement.py [-h|--help] [-d] [(-t|--tiles) (GMT|OSM)]
"""


import os
import pyslip.tkinter_error as tkinter_error
try:
    import wx
except ImportError:
    msg = 'Sorry, you must install wxPython'
    tkinter_error.tkinter_error(msg)

import pyslip
import pyslip.log as log


######
# Various demo constants
######

# demo name/version
DemoName = 'Test polyline placement, pySlip %s' % pyslip.__version__
DemoVersion = '1.0'

# initial values
InitialViewLevel = 4
InitialViewPosition = (145.0, -20.0)

# tiles info
TileDirectory = 'tiles'
MinTileLevel = 0

# the number of decimal places in a lon/lat display
LonLatPrecision = 3

# startup size of the application
DefaultAppSize = (1000, 700)

# general defaults
DefaultWidth = 5
DefaultColour = 'red'

# initial values in map-relative LayerControl
DefaultPlacement = 'ne'
DefaultX = 145.0
DefaultY = -20.0
DefaultOffsetX = 0
DefaultOffsetY = 0

# initial values in view-relative LayerControl
DefaultViewPlacement = 'ne'
DefaultViewX = 0
DefaultViewY = 0
DefaultViewOffsetX = 0
DefaultViewOffsetY = 0

# polyline map- and view-relative data
PolyPoints = [(140.0,-17.5), (144.0,-19.0), (142.5,-15.0), (147.5,-15.0),
              (146.0,-19.0), (150.0,-17.5), (150.0,-22.5), (146.0,-21.0),
              (147.5,-25.0), (142.5,-25.0), (144.0,-21.0), (140.0,-22.5)]

PolyViewPoints = [(-100,-50), (-20,-20), (-50,-100), (50,-100),
                  (20,-20), (100,-50), (100,50), (20,20),
                  (50,100), (-50,100), (-20,20), (-100,50)]

######
# Various GUI layout constants
######

# sizes of various spacers
HSpacerSize = (0,1)         # horizontal in application screen
VSpacerSize = (1,1)         # vertical in control pane

# border width when packing GUI elements
PackBorder = 0


###############################################################################
# Override the wx.TextCtrl class to add read-only style and background colour
###############################################################################

# background colour for the 'read-only' text field
ControlReadonlyColour = '#ffffcc'

class ROTextCtrl(wx.TextCtrl):
    """Override the wx.TextCtrl widget to get read-only text control which
    has a distinctive background colour."""

    def __init__(self, parent, value, tooltip='', *args, **kwargs):
        wx.TextCtrl.__init__(self, parent, wx.ID_ANY, value=value,
                             style=wx.TE_READONLY, *args, **kwargs)
        self.SetBackgroundColour(ControlReadonlyColour)
        self.SetToolTip(wx.ToolTip(tooltip))

###############################################################################
# Override the wx.StaticBox class to show our style
###############################################################################

class AppStaticBox(wx.StaticBox):

    def __init__(self, parent, label, *args, **kwargs):
        if 'style' not in kwargs:
            kwargs['style'] = wx.NO_BORDER
        wx.StaticBox.__init__(self, parent, wx.ID_ANY, label, *args, **kwargs)

###############################################################################
# Class for a LayerControl widget.
#
# This is used to control each type of layer, whether map- or view-relative.
###############################################################################

myEVT_DELETE = wx.NewEventType()
myEVT_UPDATE = wx.NewEventType()

EVT_DELETE = wx.PyEventBinder(myEVT_DELETE, 1)
EVT_UPDATE = wx.PyEventBinder(myEVT_UPDATE, 1)

class LayerControlEvent(wx.PyCommandEvent):
    """Event sent when a LayerControl is changed."""

    def __init__(self, eventType, id):
        wx.PyCommandEvent.__init__(self, eventType, id)

class LayerControl(wx.Panel):

    def __init__(self, parent, title,
                 placement=DefaultPlacement, width=DefaultWidth,
                 colour=DefaultColour, offset_x=0, offset_y=0, **kwargs):
        """Initialise a LayerControl instance.

        parent       reference to parent object
        title        text to show in static box outline around control
        placement    placement string for object
        width        width in pixels of the drawn polygon
        colour       sets the colour of the polygon outline
        offset_x     X offset of object
        offset_y     Y offset of object
        **kwargs     keyword args for Panel
        """

        # save parameters
        self.v_placement = placement
        self.v_width = width
        self.v_colour = colour
        self.v_offset_x = offset_x
        self.v_offset_y = offset_y

        # create and initialise the base panel
        wx.Panel.__init__(self, parent=parent, id=wx.ID_ANY, **kwargs)
        self.SetBackgroundColour(wx.WHITE)

        # create the widget
        box = AppStaticBox(self, title)
        sbs = wx.StaticBoxSizer(box, orient=wx.VERTICAL)
        gbs = wx.GridBagSizer(vgap=2, hgap=2)

        # row 0
        row = 0
        label = wx.StaticText(self, wx.ID_ANY, 'placement: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        choices = ['nw', 'cn', 'ne', 'ce', 'se', 'cs', 'sw', 'cw', 'cc', 'none']
        style=wx.CB_DROPDOWN|wx.CB_READONLY
        self.placement = wx.ComboBox(self, value=self.v_placement,
                                     choices=choices, style=style)
        gbs.Add(self.placement, (row,1), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.EXPAND))

        label = wx.StaticText(self, wx.ID_ANY, 'width: ')
        gbs.Add(label, (row,2), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        choices = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10']
        style=wx.CB_DROPDOWN|wx.CB_READONLY
        self.width = wx.ComboBox(self, value=self.v_width,
                                 choices=choices, style=style)
        gbs.Add(self.width, (row,3),
                border=0, flag=(wx.ALIGN_CENTER_VERTICAL|wx.EXPAND))

        # row 1
        row += 1
        label = wx.StaticText(self, wx.ID_ANY, 'colour: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.polycolour = wx.Button(self, label='')
        self.polycolour.SetBackgroundColour(self.v_colour)
        gbs.Add(self.polycolour, (row,1), border=0, flag=wx.EXPAND)

        # row 2
        row += 1
        label = wx.StaticText(self, wx.ID_ANY, 'offset_x: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.offset_x = wx.TextCtrl(self, value=str(self.v_offset_x))
        gbs.Add(self.offset_x, (row,1), border=0, flag=wx.EXPAND)

        label = wx.StaticText(self, wx.ID_ANY, '  offset_y: ')
        gbs.Add(label, (row,2), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.offset_y = wx.TextCtrl(self, value=str(self.v_offset_y))
        gbs.Add(self.offset_y, (row,3), border=0, flag=wx.EXPAND)

        # row 3
        row += 1
        delete_button = wx.Button(self, label='Remove')
        gbs.Add(delete_button, (row,1), border=10, flag=wx.EXPAND)
        update_button = wx.Button(self, label='Update')
        gbs.Add(update_button, (row,3), border=10, flag=wx.EXPAND)

        sbs.Add(gbs)
        self.SetSizer(sbs)
        sbs.Fit(self)

        self.polycolour.Bind(wx.EVT_BUTTON, self.onPolyColour)
        delete_button.Bind(wx.EVT_BUTTON, self.onDelete)
        update_button.Bind(wx.EVT_BUTTON, self.onUpdate)

    def onPolyColour(self, event):
        """Change polyline colour."""

        colour = self.polycolour.GetBackgroundColour()
        wxcolour = wx.ColourData()
        wxcolour.SetColour(colour)

        dialog = wx.ColourDialog(self, data=wxcolour)
        dialog.GetColourData().SetChooseFull(True)
        new_colour = None
        if dialog.ShowModal() == wx.ID_OK:
            new_colour = dialog.GetColourData().Colour
        dialog.Destroy()

        if new_colour:
            self.polycolour.SetBackgroundColour(new_colour)

    def onDelete(self, event):
        """Remove object from map."""

        event = LayerControlEvent(myEVT_DELETE, self.GetId())
        self.GetEventHandler().ProcessEvent(event)

    def onUpdate(self, event):
        """Update object on map."""

        event = LayerControlEvent(myEVT_UPDATE, self.GetId())

        event.placement = self.placement.GetValue()
        event.width = int(self.width.GetValue())
        event.colour = self.polycolour.GetBackgroundColour()
        event.offset_x = self.offset_x.GetValue()
        event.offset_y = self.offset_y.GetValue()

        self.GetEventHandler().ProcessEvent(event)

################################################################################
# The main application frame
################################################################################

class AppFrame(wx.Frame):
    def __init__(self, tile_dir=TileDirectory, levels=None):
        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title='%s, test version %s' % (DemoName, DemoVersion))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        self.tile_directory = tile_dir
        self.tile_source = Tiles(tile_dir, levels)

        # build the GUI
        self.make_gui(self.panel)

        # set initial view position
        self.map_level.SetLabel('%d' % InitialViewLevel)
        wx.CallAfter(self.final_setup, InitialViewLevel, InitialViewPosition)

        # force pyslip initialisation
        self.pyslip.OnSize()

        # finally, set up application window position
        self.Centre()

        # initialise state variables
        self.poly_layer = None
        self.poly_view_layer = None

        # finally, bind pySlip events to handlers
        self.pyslip.Bind(pyslip.EVT_PYSLIP_POSITION, self.handle_position_event)
        self.pyslip.Bind(pyslip.EVT_PYSLIP_LEVEL, self.handle_level_change)

#####
# Build the GUI
#####

    def make_gui(self, parent):
        """Create application GUI."""

        # start application layout
        all_display = wx.BoxSizer(wx.HORIZONTAL)
        parent.SetSizer(all_display)

        # put map view in left of horizontal box
        sl_box = self.make_gui_view(parent)
        all_display.Add(sl_box, proportion=1, border=0, flag=wx.EXPAND)

        # small spacer here - separate view and controls
        all_display.AddSpacer(HSpacerSize)

        # add controls to right of spacer
        controls = self.make_gui_controls(parent)
        all_display.Add(controls, proportion=0, border=0)

        parent.SetSizerAndFit(all_display)

    def make_gui_view(self, parent):
        """Build the map view widget

        parent  reference to the widget parent

        Returns the static box sizer.
        """

        # create gui objects
        sb = AppStaticBox(parent, '')
        self.pyslip = pyslip.PySlip(parent, tile_src=self.tile_source,
                                    min_level=MinTileLevel,
                                    tilesets=['./tilesets'])

        # lay out objects
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(self.pyslip, proportion=1, border=0, flag=wx.EXPAND)

        return box

    def make_gui_controls(self, parent):
        """Build the 'controls' part of the GUI

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # all controls in vertical box sizer
        controls = wx.BoxSizer(wx.VERTICAL)

        # add the map level in use widget
        level = self.make_gui_level(parent)
        controls.Add(level, proportion=0, flag=wx.EXPAND|wx.ALL)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # add the mouse position feedback stuff
        mouse = self.make_gui_mouse(parent)
        controls.Add(mouse, proportion=0, flag=wx.EXPAND|wx.ALL)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # controls for map-relative object layer
        self.poly = self.make_gui_poly(parent)
        controls.Add(self.poly, proportion=0, flag=wx.EXPAND|wx.ALL)
        self.poly.Bind(EVT_DELETE, self.polyDelete)
        self.poly.Bind(EVT_UPDATE, self.polyUpdate)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # controls for view-relative text layer
        self.poly_view = self.make_gui_poly_view(parent)
        controls.Add(self.poly_view, proportion=0, flag=wx.EXPAND|wx.ALL)
        self.poly_view.Bind(EVT_DELETE, self.polyViewDelete)
        self.poly_view.Bind(EVT_UPDATE, self.polyViewUpdate)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        return controls

    def make_gui_level(self, parent):
        """Build the control that shows the level.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create objects
        txt = wx.StaticText(parent, wx.ID_ANY, 'Level: ')
        self.map_level = wx.StaticText(parent, wx.ID_ANY, ' ')

        # lay out the controls
        sb = AppStaticBox(parent, 'Map level')
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(txt, border=PackBorder, flag=(wx.ALIGN_CENTER_VERTICAL
                                     |wx.ALIGN_RIGHT|wx.LEFT))
        box.Add(self.map_level, proportion=0, border=PackBorder,
                flag=wx.RIGHT|wx.TOP)

        return box

    def make_gui_mouse(self, parent):
        """Build the mouse part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create objects
        txt = wx.StaticText(parent, wx.ID_ANY, 'Lon/Lat: ')
        self.mouse_position = ROTextCtrl(parent, '', size=(150,-1),
                                         tooltip=('Shows the mouse '
                                                  'longitude and latitude '
                                                  'on the map'))

        # lay out the controls
        sb = AppStaticBox(parent, 'Mouse position')
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(txt, border=PackBorder, flag=(wx.ALIGN_CENTER_VERTICAL
                                     |wx.ALIGN_RIGHT|wx.LEFT))
        box.Add(self.mouse_position, proportion=1, border=PackBorder,
                flag=wx.RIGHT|wx.TOP|wx.BOTTOM)

        return box

    def make_gui_poly(self, parent):
        """Build the polygon map-relative part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create widgets
        poly_obj = LayerControl(parent, 'Polygon, map-relative',
                                placement=DefaultPlacement,
                                width=str(DefaultWidth),
                                colour=DefaultColour,
                                offset_x=DefaultOffsetX,
                                offset_y=DefaultOffsetY)

        return poly_obj

    def make_gui_poly_view(self, parent):
        """Build the view-relative polygon part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create widgets
        poly_obj = LayerControl(parent, 'Polygon, view-relative',
                                 placement=DefaultPlacement,
                                 width=str(DefaultWidth),
                                 colour=DefaultColour,
                                 offset_x=DefaultViewOffsetX,
                                 offset_y=DefaultViewOffsetY)

        return poly_obj

    ######
    # event handlers
    ######

##### map-relative polygon layer

    def polyUpdate(self, event):
        """Display updated polygon."""

        if self.poly_layer:
            self.pyslip.DeleteLayer(self.poly_layer)

        # convert values to sanity for layer attributes
        placement = event.placement
        if placement == 'none':
            placement= ''

        width = event.width
        colour = event.colour

        off_x = event.offset_x
        if not off_x:
            off_x = 0
        try:
            off_x = int(off_x)
        except ValueError:
            off_x = 0

        off_y = event.offset_y
        if not off_y:
            off_y = 0
        try:
            off_y = int(off_y)
        except ValueError:
            off_y = 0

        poly_data = [(PolyPoints, {'placement': placement,
                                   'width': width,
                                   'colour': colour,
                                   'offset_x': off_x,
                                   'offset_y': off_y})]
        self.poly_layer = self.pyslip.AddPolygonLayer(poly_data, map_rel=True,
                                                      visible=True,
                                                      name='<poly_layer>')

    def polyDelete(self, event):
        """Delete the polygon map-relative layer."""

        if self.poly_layer:
            self.pyslip.DeleteLayer(self.poly_layer)
        self.poly_layer = None

##### view-relative polygon layer

    def polyViewUpdate(self, event):
        """Display updated polygon layer."""

        if self.poly_view_layer:
            self.pyslip.DeleteLayer(self.poly_view_layer)

        # convert values to sanity for layer attributes
        placement = event.placement
        if placement == 'none':
            placement= ''

        width = event.width
        colour = event.colour

        off_x = event.offset_x
        if not off_x:
            off_x = 0
        off_x = int(off_x)

        off_y = event.offset_y
        if not off_y:
            off_y = 0
        off_y = int(off_y)

        # create a new polygon layer
        poly_data = [(PolyViewPoints, {'placement': placement,
                                       'width': width,
                                       'colour': colour,
                                       'offset_x': off_x,
                                       'offset_y': off_y})]
        self.poly_view_layer = self.pyslip.AddPolygonLayer(poly_data,
                                                           map_rel=False,
                                                           visible=True,
                                                           name='<poly_layer>')

    def polyViewDelete(self, event):
        """Delete the polygon view-relative layer."""

        if self.poly_view_layer:
            self.pyslip.DeleteLayer(self.poly_view_layer)
        self.poly_view_layer = None

    def final_setup(self, level, position):
        """Perform final setup.

        level     zoom level required
        position  position to be in centre of view
        """

        self.pyslip.GotoLevelAndPosition(level, position)

    ######
    # Exception handlers
    ######

    def handle_position_event(self, event):
        """Handle a pySlip POSITION event."""

        posn_str = ''
        if event.mposn:
            (lon, lat) = event.mposn
            posn_str = ('%.*f / %.*f' % (LonLatPrecision, lon,
                                         LonLatPrecision, lat))

        self.mouse_position.SetValue(posn_str)

    def handle_level_change(self, event):
        """Handle a pySlip LEVEL event."""

        self.map_level.SetLabel('%d' % event.level)

###############################################################################

if __name__ == '__main__':
    import sys
    import getopt
    import traceback

#vvvvvvvvvvvvvvvvvvvvv test code - can go away once __init__.py works
#    DefaultTilesets = 'tilesets'
#    CurrentPath = os.path.dirname(os.path.abspath(__file__))
#
#    sys.path.append(os.path.join(CurrentPath, DefaultTilesets))
#
#    log(str(sys.path))
#^^^^^^^^^^^^^^^^^^^^^ test code - can go away once __init__.py works

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        print(msg)
        log(msg)
        tkinter_error.tkinter_error(msg)
        sys.exit(1)

    def usage(msg=None):
        if msg:
            print(('*'*80 + '\n%s\n' + '*'*80) % msg)
        print(__doc__)


    # plug our handler into the python system
    sys.excepthook = excepthook

    # decide which tiles to use, default is GMT
    argv = sys.argv[1:]

    try:
        (opts, args) = getopt.getopt(argv, 'dht:', ['debug', 'help', 'tiles='])
    except getopt.error:
        usage()
        sys.exit(1)

    tile_source = 'GMT'
    debug = False
    for (opt, param) in opts:
        if opt in ['-h', '--help']:
            usage()
            sys.exit(0)
        elif opt in ['-d', '--debug']:
            debug = True
        elif opt in ('-t', '--tiles'):
            tile_source = param
    tile_source = tile_source.lower()

    # set up the appropriate tile source
    if tile_source == 'gmt':
        from pyslip.gmt_local_tiles import GMTTiles as Tiles
        tile_dir = 'gmt_tiles'
    elif tile_source == 'osm':
        from pyslip.osm_tiles import OSMTiles as Tiles
        tile_dir = 'osm_tiles'
    else:
        usage('Bad tile source: %s' % tile_source)
        sys.exit(3)

    # start wxPython app
    app = wx.App()

    app_frame = AppFrame(tile_dir=tile_dir)
    app_frame.Show()

    if debug:
        import wx.lib.inspection
        wx.lib.inspection.InspectionTool().Show()

    app.MainLoop()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test the local GMT tiles code.

Requires a wxPython application to be created before use.
If we can create a bitmap without wxPython, we could remove this dependency.
"""

import os
import glob
import pickle
import wx
import pyslip.gmt_local_tiles as gmt_local_tiles

import unittest
import shutil
from wx.lib.embeddedimage import PyEmbeddedImage


# where the pre-generated GMT tiles are
TilesDir = './gmt_tiles'

DefaultAppSize = (512, 512)
DemoName = 'GMT Tiles Cache Test'
DemoVersion = '0.1'


class AppFrame(wx.Frame):

    def __init__(self):
        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title='%s %s' % (DemoName, DemoVersion))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()
        self.Bind(wx.EVT_CLOSE, self.onClose)

        unittest.main()

    def onClose(self, event):
        self.Destroy()

class TestGMTTiles(unittest.TestCase):

    # for GMT tiles
    TileWidth = 256
    TileHeight = 256

    def testSimple(self):
        """Simple tests."""

        # read all tiles in all rows of all levels
        cache = gmt_local_tiles.GMTTiles(tiles_dir=TilesDir)
        for level in cache.levels:
            cache.UseLevel(level)
            info = cache.GetInfo(level)
            if info:
                width_px = self.TileWidth * cache.num_tiles_x
                height_px = self.TileHeight * cache.num_tiles_y
                ppd_x = cache.ppd_x
                ppd_y = cache.ppd_y
                num_tiles_width = int(width_px / self.TileWidth)
                num_tiles_height = int(height_px / self.TileHeight)
                for x in range(num_tiles_width):
                    for y in range(num_tiles_height):
                        bmp = cache.GetTile(x, y)
                        msg = "Can't find tile (%d,%d,%d)!?" % (level, x, y)
                        self.failIf(bmp is None, msg)

    def testErrors(self):
        """Test possible errors."""

        # try to use on-disk cache that doesn't exist
        with self.assertRaises(IOError):
            cache = gmt_local_tiles.GMTTiles(tiles_dir='_=XYZZY=_')

        # check that using level outside map levels returns None
        cache = gmt_local_tiles.GMTTiles(tiles_dir=TilesDir)
        level = cache.levels[-1] + 1      # get level # that DOESN'T exist
        msg = "Using bad level (%d) didn't raise exception?" % level
        with self.assertRaises(Exception, msg=msg):
            cache.UseLevel(level)

        # check that reading tile outside map returns None
        cache = gmt_local_tiles.GMTTiles(tiles_dir=TilesDir)
        level = cache.levels[0] # known good level
        cache.UseLevel(level)
        width_px = self.TileWidth * cache.num_tiles_x
        height_px = self.TileHeight * cache.num_tiles_y
        ppd_x = cache.ppd_x
        ppd_y = cache.ppd_y
        num_tiles_width = int(width_px / self.TileWidth)
        num_tiles_height = int(height_px / self.TileHeight)
        bmp = cache.GetTile(num_tiles_width, num_tiles_height)
        self.assertTrue(bmp is None,
                        'Using bad coords (%d,%d) got bmp=%s'
                        % (num_tiles_width, num_tiles_height, str(bmp)))

    def XtestConvert(self):
        """Test geo2map conversions.

        This is normally turned off as it is a "by hand" sort of check.
        """

        cache = gmt_local_tiles.GMTTiles(tiles_dir=TilesDir)

        # get tile covering Greenwich observatory
#        xgeo = -0.0005  # Greenwich observatory
#        ygeo = 51.4768534
        xgeo = 7.605916 # Deutsches Eck
        ygeo = 50.364444
        for level in [0, 1, 2, 3, 4]:
            cache.UseLevel(level)
            (xtile, ytile) = cache.Geo2Tile(xgeo, ygeo)
            bmp = cache.GetTile(int(xtile), int(ytile))

            pt_px_x = int((xtile - int(xtile)) * cache.tile_size_x)
            pt_px_y = int((ytile - int(ytile)) * cache.tile_size_y)

            dc = wx.MemoryDC()
            dc.SelectObject(bmp)
            text = "o"
            (tw, th) = dc.GetTextExtent(text)
            dc.DrawText(text, pt_px_x-tw/2,  pt_px_y-th/2)
            dc.SelectObject(wx.NullBitmap)

            bmp.SaveFile('xyzzy_%d.jpg' % level, wx.BITMAP_TYPE_JPEG)

app = wx.App()
app_frame = AppFrame()
app_frame.Show()
app.MainLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module to test the point_segment_distance() and point_near_polyline() functions.

A polyline is a series of points [(x1,y1), (x2,y2), ...].
"""

import math


def close_to_polyline(polyline, point, delta):
    """Decide if point is within 'delta' of the given polyline.

    polyline  iterable of (x, y) point tuples
    point     point (x, y)
    delta     maximum distance before 'not close enough'

    Returns True if point within delta distance of the polyline.
    """

    last_pp = polyline[0]
    for pp in polyline[1:]:
        if point_segment_distance(point, last_pp, pp) <= delta:
            return True
        last_pp = pp

    return False

def point_segment_distance_OLD(point, s1, s2):
    """Get distance from a point to segment (s1, s2).

    point   tuple (x, y)
    s1, s2  tuples (x, y) of segment endpoints

    Return the distance squared.
    """

    (ptx, pty) = point
    (s1x, s1y) = s1
    (s2x, s2y) = s2

    px = s2x - s1x
    py = s2y - s1y

    something = px*px + py*py

    u = ((ptx - s1x)*px + (pty - s1y)*py) / float(something)

    if u > 1:
        u = 1
    elif u < 0:
        u = 0

    x = s1x + u*px
    y = s1y + u*py

    dx = x - ptx
    dy = y - pty

    dist = math.sqrt(dx*dx + dy*dy)

    return dist

def point_segment_distance(point, s1, s2):
    """Get distance from a point to segment (s1, s2).

    point   tuple (x, y)
    s1, s2  tuples (x, y) of segment endpoints

    Return the distance squared.
    """

    (ptx, pty) = point
    (s1x, s1y) = s1
    (s2x, s2y) = s2

    px = s2x - s1x
    py = s2y - s1y

    u = ((ptx - s1x)*px + (pty - s1y)*py) / float(px**2 + py**2)

    if u > 1:
        u = 1
    elif u < 0:
        u = 0

    dx = s1x + u*px - ptx
    dy = s1y + u*py - pty

    return dx**2 + dy**2


if __name__ == '__main__':
    import unittest
    import time

    Loops = 10000000

    start = time.time()
    for i in xrange(Loops):
        a = point_segment_distance_OLD((1,1), (0,1), (0,-1))
    delta = time.time() - start
    print('Orig: %.2fs' % delta)

    start = time.time()
    for i in xrange(Loops):
        a = point_segment_distance((1,1), (0,1), (0,-1))
    delta = time.time() - start
    print('   Y: %.2fs' % delta)

    TestFunc = point_segment_distance

    NumPlaces = 11


    class TestAssumptions(unittest.TestCase):                                        
                                                                                         
        def test_simple1(self):                                                    
            """Check simple point/segment cases.
            
            This case is a vertical segment of 2 units length.
            All points should be 1 unit from the segment.
            """

            seg1 = (0,1)
            seg2 = (0,-1)
            points = [(1,1),(1,0),(1,-1),(0,-2),(-1,-1),(-1,0),(-1,1),(0,2)]
            expected = 1.0**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertEqual(dist, expected, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertEqual(dist, expected, msg=msg)

        def test_simple2(self):                                                    
            """Check simple point/segment cases.
            
            This case is a horizontal segment of 2 units length.
            All points should be 1 unit from the segment.
            """

            seg1 = (-1,0)
            seg2 = (1,0)
            points = [(0,1),(1,1),(2,0),(1,-1),(0,-1),(-1,-1),(-2,0),(-1,1)]
            expected = 1.0**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertEqual(dist, expected, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertEqual(dist, expected, msg=msg)

        def test_simple3(self):                                                    
            """Check simple point/segment cases.
            
            This case is a slant segment.
            All points should be about 0.707 units from the segment.
            """

            seg1 = (-1,-1)
            seg2 = (1,1)
            points = [(1,0),(0,-1),(-1,0),(0,1)]
            expected = 0.707106781187**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

        def test_simple4(self):                                                    
            """Check simple point/segment cases.
            
            This case is an opposite slant segment.
            All points should be about 0.707 units from the segment.
            """

            seg1 = (-1,1)
            seg2 = (1,-1)
            points = [(1,0),(0,-1),(-1,0),(0,1)]
            expected = 0.707106781187**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

        def test_simple5(self):                                                    
            """Check simple point/segment cases.
            
            This case is a horizontal segment.
            All points should be on the line, ie distance is 0.0.
            """

            seg1 = (-1,0)
            seg2 = (1,0)
            points = [seg1, (0,0), seg2]
            expected = 0.0**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

        def test_simple6(self):                                                    
            """Check simple point/segment cases.
            
            This case is a vertical segment.
            All points should be on the line, ie distance is 0.0.
            """

            seg1 = (0,1)
            seg2 = (0,-1)
            points = [seg1, (0,0), seg2]
            expected = 0.0**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

        def test_simple7(self):                                                    
            """Check simple point/segment cases.
            
            This case is a slant segment.
            All points should be on the line, ie distance is 0.0.
            """

            seg1 = (1,1)
            seg2 = (-1,-1)
            points = [seg1, (0,0), seg2]
            expected = 0.0**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

        def test_simple8(self):                                                    
            """Check simple point/segment cases.  Not straddling axes.
            NE quadrant.  Test horizontal, vertical and slant cases.
            """

            seg1 = (2,2)
            seg2 = (3,2)
            points = [(1,2),(2,3),(2.5,3),(3,3),(4,2),(3,1),(2.3,1),(2,1)]
            expected = 1.0**2

            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)

            # reverse segment, test again
            (seg1, seg2) = (seg2, seg1)
            for point in points:
                dist = TestFunc(point, seg1, seg2)
                msg = ('Segment [%s,%s], point %s distance should be %s, got %s'
                        % (str(seg1), str(seg2), str(point),
                            str(expected), str(dist)))

                self.assertAlmostEqual(dist, expected, places=NumPlaces, msg=msg)


    suite = unittest.makeSuite(TestAssumptions,'test')                           
    runner = unittest.TextTestRunner()                                           
    runner.run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
An extended dictionary offering limited LRU entries in the dictionary
and an interface to an unlimited backing store.

https://github.com/rzzzwilson/pyCacheBack
"""


class pyCacheBack(dict):
    """An LRU limited in-memory store fronting an unlimited on-disk store."""

    # default maximum number of key/value pairs for pyCacheBack
    DefaultMaxLRU = 1000

    # default path to tiles directory
    DefaultTilesDir = 'tiles'

    def __init__(self, *args, **kwargs):
        self._lru_list = []
        self._max_lru = kwargs.pop('max_lru', self.DefaultMaxLRU)
        self._tiles_dir = kwargs.pop('tiles_dir', self.DefaultTilesDir)
        super(pyCacheBack, self).__init__(*args, **kwargs)

    def __getitem__(self, key):
        if key in self:
            value = super(pyCacheBack, self).__getitem__(key)
        else:
            value = self._get_from_back(key)
        self._reorder_lru(key)
        return value

    def __setitem__(self, key, value):
        super(pyCacheBack, self).__setitem__(key, value)
        self._put_to_back(key, value)
        self._reorder_lru(key)
        self._enforce_lru_size()

    def __delitem__(self, key):
        super(pyCacheBack, self).__delitem__(key)
        self._reorder_lru(key, remove=True)

    def clear(self):
        super(pyCacheBack, self).clear()
        self._lru_list = []

    def pop(self, *args):
        k = args[0]
        try:
            self._lru_list.remove(k)
        except ValueError:
            pass
        return super(pyCacheBack, self).pop(*args)

    def popitem(self):
        kv_return = super(pyCacheBack, self).popitem()
        try:
            self._lru_list.remove(kv_return[0])
        except ValueError:
            pass
        return kv_return

    def _reorder_lru(self, key, remove=False):
        """Move key in LRU (if it exists) to 'recent' end.

        If 'remove' is True just remove from the LRU.
        """

        try:
            self._lru_list.remove(key)
        except ValueError:
            pass
        if remove:
            return
        self._lru_list.insert(0, key)

    def _enforce_lru_size(self):
        """Enforce LRU size limit in cache dictionary."""

        # if a limit was defined and we have blown it
        if self._max_lru and len(self) > self._max_lru:
            # make sure in-memory dictionary doesn't get bigger
            for key in self._lru_list[self._max_lru:]:
                super(pyCacheBack, self).__delitem__(key)
            # also truncate the LRU list
            self._lru_list = self._lru_list[:self._max_lru]

    #####
    # override the following two methods to implement the backing cache
    #####

    def _put_to_back(self, key, value):
        """Store 'value' in backing store, using 'key' to access."""

        pass

    def _get_from_back(self, key):
        """Retrieve value for 'key' from backing storage.

        Raises KeyError if key not in backing storage.
        """

        raise KeyError




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test PySlip view-relative polygons.

Usage: test_maprel_image.py [-h] [-t (OSM|GMT)]
"""


import wx
import pyslip


######
# Various demo constants
######

DefaultAppSize = (600, 400)

MinTileLevel = 0
InitViewLevel = 2
InitViewPosition = (133.87, -23.7)      # Alice Springs

arrow_cn = ((0,0),(10,10),(5,10),(5,20),(-5,20),(-5,10),(-10,10))
arrow_ne = ((-1,0),(-1,10),(-4,8),(-9,13),(-14,8),(-9,3),(-11,0))
arrow_ce = ((-1,0),(-11,10),(-11,5),(-21,5),(-21,-5),(-11,-5),(-11,-10))
arrow_se = ((-1,-1),(-1,-10),(-4,-8),(-9,-13),(-14,-8),(-9,-3),(-11,-1))
arrow_cs = ((0,-1),(-10,-11),(-5,-11),(-5,-21),(5,-21),(5,-11),(10,-11))
arrow_sw = ((0,-1),(0,-10),(3,-8),(8,-13),(13,-8),(8,-3),(10,-1))
arrow_cw = ((0,0),(10,10),(10,5),(20,5),(20,-5),(10,-5),(10,-10))
arrow_nw = ((0,0),(0,10),(3,8),(8,13),(13,8),(8,3),(10,0))
filled_poly = ((-100,100),(-100,-100),(0,150),(100,-100),(100,100))

PolyViewData = [(arrow_cn, {'placement': 'cn'}),
                (arrow_ne, {'placement': 'ne'}),
                (arrow_ce, {'placement': 'ce'}),
                (arrow_se, {'placement': 'se'}),
                (arrow_cs, {'placement': 'cs'}),
                (arrow_sw, {'placement': 'sw'}),
                (arrow_cw, {'placement': 'cw'}),
                (arrow_nw, {'placement': 'nw'}),
                (filled_poly, {'placement': 'cc', 'width': 8,
                               'fillcolour': '#ff000020',
                               'colour': '#00ff0040',
                               'filled': True}),
               ] 

################################################################################
# The main application frame
################################################################################

class TestFrame(wx.Frame):
    def __init__(self, tile_dir):
        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title=('PySlip %s - view-relative polygon test'
                                 % pyslip.__version__))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        # create the tile source object
        self.tile_src = Tiles(tile_dir)

        # build the GUI                                                                                                             
        box = wx.BoxSizer(wx.HORIZONTAL)                                                                                            
        self.panel.SetSizer(box)                                                                                                    
        self.pyslip = pyslip.PySlip(self.panel, tile_src=self.tile_src,                                                             
                                    min_level=MinTileLevel)                                                                         
        box.Add(self.pyslip, proportion=1, border=1, flag=wx.EXPAND)                                                                
        self.panel.SetSizer(box)                                                                                                    
        self.panel.Layout()                                                                                                         
        self.Centre()                                                                                                               
        self.Show(True)              

        # set initial view position
        self.pyslip.GotoLevelAndPosition(InitViewLevel, InitViewPosition)

        # add test test layer
        self.text_layer = self.pyslip.AddPolygonLayer(PolyViewData,
                                                      map_rel=False,
                                                      name='<poly_map_layer>',
                                                      offset_x=0, offset_y=0,
                                                      closed=True)

################################################################################

if __name__ == '__main__':
    import sys
    import getopt
    import traceback

    # print some usage information                                                                                                  
    def usage(msg=None):                                                                                                            
        if msg:                                                                                                                     
            print(msg+'\n')                                                                                                         
        print(__doc__)        # module docstring used

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        print msg
        sys.exit(1)

    # plug our handler into the python system
    sys.excepthook = excepthook

    # decide which tiles to use, default is GMT                                                                                     
    argv = sys.argv[1:]                                                                                                             
    
    try:                                                                                                                            
        (opts, args) = getopt.getopt(argv, 'ht:', ['help', 'tiles='])                                                               
    except getopt.error:                                                                                                            
        usage()                                                                                                                     
        sys.exit(1)                                                                                                                 
    
    tile_source = 'GMT'                                                                                                             
    for (opt, param) in opts:                                                                                                       
        if opt in ['-h', '--help']:                                                                                                 
            usage()                                                                                                                 
            sys.exit(0)                                                                                                             
        elif opt in ('-t', '--tiles'):                                                                                              
            tile_source = param                                                                                                     
    tile_source = tile_source.lower()                                                                                               
    
    # set up the appropriate tile source                                                                                            
    if tile_source == 'gmt':                                                                                                        
        from pyslip.gmt_local_tiles import GMTTiles as Tiles                                                                        
        tile_dir = 'gmt_tiles'                                                                                                      
    elif tile_source == 'osm':                                                                                                      
        from pyslip.osm_tiles import OSMTiles as Tiles                                                                                     
        tile_dir = 'osm_tiles'                                                                                                      
    else:                                                                                                                           
        usage('Bad tile source: %s' % tile_source)                                                                                  
        sys.exit(3)                                                                                                                 
    
    # start wxPython app
    app = wx.App()
    TestFrame(tile_dir).Show()
    app.MainLoop()




#!/usr/bin/env python
# -*- coding= utf-8 -*-

"""
Program to test image map-relative and view-relative placement.
Select which to show and experiment with placement parameters.

Usage: test_image_placement.py [-h|--help] [-d] [(-t|--tiles) (GMT|OSM)]
"""


import os
import pyslip.tkinter_error as tkinter_error
try:
    import wx
except ImportError:
    msg = 'Sorry, you must install wxPython'
    tkinter_error.tkinter_error(msg)

import pyslip
import pyslip.log as log


######
# Various demo constants
######

# demo name/version
DemoName = 'Test image placement, pySlip %s' % pyslip.__version__
DemoVersion = '1.0'

# initial values
InitialViewLevel = 4
InitialViewPosition = (145.0, -20.0)

# tiles info
TileDirectory = 'tiles'
MinTileLevel = 0

# the number of decimal places in a lon/lat display
LonLatPrecision = 3

# startup size of the application
DefaultAppSize = (1000, 700)

# initial values in map-relative LayerControl
DefaultFilename = 'graphics/shipwreck.png'
DefaultPlacement = 'ne'
DefaultPointColour = 'red'
DefaultPointRadius = 3
DefaultX = 145.0
DefaultY = -20.0
DefaultOffsetX = 0
DefaultOffsetY = 0

# initial values in view-relative LayerControl
DefaultViewFilename = 'graphics/compass_rose.png'
DefaultViewPlacement = 'ne'
PointRadiusChoices = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10']
DefaultViewX = 0
DefaultViewY = 0
DefaultViewOffsetX = 0
DefaultViewOffsetY = 0

######
# Various GUI layout constants
######

# sizes of various spacers
HSpacerSize = (0,1)         # horizontal in application screen
VSpacerSize = (1,1)         # vertical in control pane

# border width when packing GUI elements
PackBorder = 0


###############################################################################
# Override the wx.TextCtrl class to add read-only style and background colour
###############################################################################

# background colour for the 'read-only' text field
ControlReadonlyColour = '#ffffcc'

class ROTextCtrl(wx.TextCtrl):
    """Override the wx.TextCtrl widget to get read-only text control which
    has a distinctive background colour."""

    def __init__(self, parent, value, tooltip='', *args, **kwargs):
        wx.TextCtrl.__init__(self, parent, wx.ID_ANY, value=value,
                             style=wx.TE_READONLY, *args, **kwargs)
        self.SetBackgroundColour(ControlReadonlyColour)
        self.SetToolTip(wx.ToolTip(tooltip))

###############################################################################
# Override the wx.StaticBox class to show our style
###############################################################################

class AppStaticBox(wx.StaticBox):

    def __init__(self, parent, label, *args, **kwargs):
        if 'style' not in kwargs:
            kwargs['style'] = wx.NO_BORDER
        wx.StaticBox.__init__(self, parent, wx.ID_ANY, label, *args, **kwargs)

###############################################################################
# Class for a LayerControl widget.
#
# This is used to control each type of layer, whether map- or view-relative.
###############################################################################

myEVT_DELETE = wx.NewEventType()
myEVT_UPDATE = wx.NewEventType()

EVT_DELETE = wx.PyEventBinder(myEVT_DELETE, 1)
EVT_UPDATE = wx.PyEventBinder(myEVT_UPDATE, 1)

class LayerControlEvent(wx.PyCommandEvent):
    """Event sent when a LayerControl is changed."""

    def __init__(self, eventType, id):
        wx.PyCommandEvent.__init__(self, eventType, id)

class LayerControl(wx.Panel):

    def __init__(self, parent, title, filename='', placement=DefaultPlacement,
                 pointradius=DefaultPointRadius, pointcolour=DefaultPointColour,
                 x=0, y=0, offset_x=0, offset_y=0, **kwargs):
        """Initialise a LayerControl instance.

        parent       reference to parent object
        title        text to show in static box outline
        filename     filename of image to show
        placement    placement string for image
        pointradius  radius of the image point
        pointcolour  colour of the image point
        x, y         X and Y coords
        offset_x     X offset of image
        offset_y     Y offset of image
        **kwargs     keyword args for Panel
        """

        # create and initialise the base panel
        wx.Panel.__init__(self, parent=parent, id=wx.ID_ANY, **kwargs)
        self.SetBackgroundColour(wx.WHITE)

        self.v_filename = filename
        self.v_placement = placement
        self.v_pointradius = str(pointradius)
        self.v_pointcolour = pointcolour
        self.v_x = x
        self.v_y = y
        self.v_offset_x = offset_x
        self.v_offset_y = offset_y

        box = AppStaticBox(self, title)
        sbs = wx.StaticBoxSizer(box, orient=wx.VERTICAL)
        gbs = wx.GridBagSizer(vgap=2, hgap=2)

        # row 0
        row = 0
        label = wx.StaticText(self, wx.ID_ANY, 'filename: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.filename = ROTextCtrl(self, self.v_filename)
        gbs.Add(self.filename, (row,1), span=(1,3), border=0, flag=wx.EXPAND)

        # row 1
        row += 1
        label = wx.StaticText(self, wx.ID_ANY, 'placement: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        choices = ['nw', 'cn', 'ne', 'ce', 'se', 'cs', 'sw', 'cw', 'cc', 'none']
        style=wx.CB_DROPDOWN|wx.CB_READONLY
        self.placement = wx.ComboBox(self, value=self.v_placement,
                                     choices=choices, style=style)
        gbs.Add(self.placement, (row,1), border=0)

        # row 2
        row += 1
        label = wx.StaticText(self, wx.ID_ANY, 'point radius: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        style=wx.CB_DROPDOWN|wx.CB_READONLY
        self.pointradius = wx.ComboBox(self, value=self.v_pointradius,
                                       choices=PointRadiusChoices, style=style)
        gbs.Add(self.pointradius, (row,1),
                border=0, flag=(wx.ALIGN_CENTER_VERTICAL|wx.EXPAND))
        label = wx.StaticText(self, wx.ID_ANY, 'point colour: ')
        gbs.Add(label, (row,2), border=0,
        flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.pointcolour = wx.Button(self, label='')
        self.pointcolour.SetBackgroundColour(self.v_pointcolour)
        gbs.Add(self.pointcolour, (row,3), border=0, flag=wx.EXPAND)

        # row 3
        row += 1
        label = wx.StaticText(self, wx.ID_ANY, 'x: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.x = wx.TextCtrl(self, value=str(self.v_x))
        gbs.Add(self.x, (row,1), border=0, flag=wx.EXPAND)

        label = wx.StaticText(self, wx.ID_ANY, 'y: ')
        gbs.Add(label, (row,2), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.y = wx.TextCtrl(self, value=str(self.v_y))
        gbs.Add(self.y, (row,3), border=0, flag=wx.EXPAND)

        # row 4
        row += 1
        label = wx.StaticText(self, wx.ID_ANY, 'offset_x: ')
        gbs.Add(label, (row,0), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.offset_x = wx.TextCtrl(self, value=str(self.v_offset_x))
        gbs.Add(self.offset_x, (row,1), border=0, flag=wx.EXPAND)

        label = wx.StaticText(self, wx.ID_ANY, '  offset_y: ')
        gbs.Add(label, (row,2), border=0,
                flag=(wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT))
        self.offset_y = wx.TextCtrl(self, value=str(self.v_offset_y))
        gbs.Add(self.offset_y, (row,3), border=0, flag=wx.EXPAND)

        # row 5
        row += 1
        delete_button = wx.Button(self, label='Remove')
        gbs.Add(delete_button, (row,1), border=10, flag=wx.EXPAND)
        update_button = wx.Button(self, label='Update')
        gbs.Add(update_button, (row,3), border=10, flag=wx.EXPAND)

        sbs.Add(gbs)
        self.SetSizer(sbs)
        sbs.Fit(self)

        self.filename.Bind(wx.EVT_LEFT_UP, self.onFilename)
        self.pointcolour.Bind(wx.EVT_BUTTON, self.onPointColour)
        delete_button.Bind(wx.EVT_BUTTON, self.onDelete)
        update_button.Bind(wx.EVT_BUTTON, self.onUpdate)

    def onFilename(self, event):
        """Change image filename."""

        wildcard = ("PNG files (*.png)|*.png|"
                    "JPG files (*.jpg)|*.jpg|"
                    "All files (*.*)|*.*")

        filepath = None

        dialog = wx.FileDialog(None, "Choose an image file",
                               os.getcwd(), "", wildcard, wx.OPEN)
        if dialog.ShowModal() == wx.ID_OK:
            filepath = dialog.GetPath()
        dialog.Destroy()

        if filepath:
            self.filename.SetValue(filepath)

    def onPointColour(self, event):
        """Change text colour."""

        colour = self.pointcolour.GetBackgroundColour()
        wxcolour = wx.ColourData()
        wxcolour.SetColour(colour)

        dialog = wx.ColourDialog(self, data=wxcolour)
        dialog.GetColourData().SetChooseFull(True)
        new_colour = None
        if dialog.ShowModal() == wx.ID_OK:
            new_colour = dialog.GetColourData().Colour
        dialog.Destroy()

        if new_colour:
            self.pointcolour.SetBackgroundColour(new_colour)

    def onDelete(self, event):
        """Remove image from map."""

        event = LayerControlEvent(myEVT_DELETE, self.GetId())
        self.GetEventHandler().ProcessEvent(event)

    def get_numeric_value(self, control):
        """Get numeric value of a textbox.

        control  the textbox to query

        Return numeric value as a float.
        """

        orig_value = control.GetValue()
        if not orig_value:
            value = '0'
            control.SetValue('0')
        else:
            value = orig_value

        try:
            value = float(value)
        except ValueError:
            return (orig_value, None)
        return (orig_value, value)

    def onUpdate(self, event):
        """Update image on map."""

        # get x/y/offset_x/offset_y and check valid
        (orig_x, x) = self.get_numeric_value(self.x)
        (orig_y, y) = self.get_numeric_value(self.y)
        (orig_offset_x, offset_x) = self.get_numeric_value(self.offset_x)
        (orig_offset_y, offset_y) = self.get_numeric_value(self.offset_y)

        if (x is not None and y is not None
                and offset_x is not None and offset_y is not None):
            event = LayerControlEvent(myEVT_UPDATE, self.GetId())

            event.filename = self.filename.GetValue()
            event.placement = self.placement.GetValue()
            event.radius = int(self.pointradius.GetValue())
            event.colour = self.pointcolour.GetBackgroundColour()
            event.x = self.x.GetValue()
            event.y = self.y.GetValue()
            event.offset_x = self.offset_x.GetValue()
            event.offset_y = self.offset_y.GetValue()

            self.GetEventHandler().ProcessEvent(event)
        else:
            msg = 'These controls have bad values:\n'
            if x is None:
                name = ('x:' + ' '*20)[:12]
                msg += "\t%s\t%s\n" % (name, str(orig_x))
            if y is None:
                name = ('y:' + ' '*20)[:12]
                msg += "\t%s\t%s\n" % (name, str(orig_y))
            if offset_x is None:
                name = ('offset_x:' + ' '*20)[:12]
                msg += "\t%s\t%s\n" % (name, str(orig_offset_x))
            if offset_y is None:
                name = ('offset_y:' + ' '*20)[:12]
                msg += "\t%s\t%s\n" % (name, str(orig_offset_y))
            wx.MessageBox(msg, 'Warning', wx.OK | wx.ICON_ERROR)


################################################################################
# The main application frame
################################################################################

class AppFrame(wx.Frame):
    def __init__(self, tile_dir=TileDirectory, levels=None):
        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title='%s, test version %s' % (DemoName, DemoVersion))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        self.tile_directory = tile_dir
        self.tile_source = Tiles(tile_dir, levels)

        # build the GUI
        self.make_gui(self.panel)

        # set initial view position
        self.map_level.SetLabel('%d' % InitialViewLevel)
        wx.CallAfter(self.final_setup, InitialViewLevel, InitialViewPosition)

        # force pyslip initialisation
        self.pyslip.OnSize()

        # finally, set up application window position
        self.Centre()

        # initialise state variables
        self.image_layer = None
        self.image_view_layer = None

        # finally, bind pySlip events to handlers
        self.pyslip.Bind(pyslip.EVT_PYSLIP_POSITION, self.handle_position_event)
        self.pyslip.Bind(pyslip.EVT_PYSLIP_LEVEL, self.handle_level_change)

#####
# Build the GUI
#####

    def make_gui(self, parent):
        """Create application GUI."""

        # start application layout
        all_display = wx.BoxSizer(wx.HORIZONTAL)
        parent.SetSizer(all_display)

        # put map view in left of horizontal box
        sl_box = self.make_gui_view(parent)
        all_display.Add(sl_box, proportion=1, border=0, flag=wx.EXPAND)

        # small spacer here - separate view and controls
        all_display.AddSpacer(HSpacerSize)

        # add controls to right of spacer
        controls = self.make_gui_controls(parent)
        all_display.Add(controls, proportion=0, border=0)

        parent.SetSizerAndFit(all_display)

    def make_gui_view(self, parent):
        """Build the map view widget

        parent  reference to the widget parent

        Returns the static box sizer.
        """

        # create gui objects
        sb = AppStaticBox(parent, '')
        self.pyslip = pyslip.PySlip(parent, tile_src=self.tile_source,
                                    min_level=MinTileLevel,
                                    tilesets=['./tilesets'])

        # lay out objects
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(self.pyslip, proportion=1, border=0, flag=wx.EXPAND)

        return box

    def make_gui_controls(self, parent):
        """Build the 'controls' part of the GUI

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # all controls in vertical box sizer
        controls = wx.BoxSizer(wx.VERTICAL)

        # add the map level in use widget
        level = self.make_gui_level(parent)
        controls.Add(level, proportion=0, flag=wx.EXPAND|wx.ALL)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # add the mouse position feedback stuff
        mouse = self.make_gui_mouse(parent)
        controls.Add(mouse, proportion=0, flag=wx.EXPAND|wx.ALL)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # controls for map-relative image layer
        self.image = self.make_gui_image(parent)
        controls.Add(self.image, proportion=0, flag=wx.EXPAND|wx.ALL)
        self.image.Bind(EVT_DELETE, self.imageDelete)
        self.image.Bind(EVT_UPDATE, self.imageUpdate)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # controls for image-relative image layer
        self.image_view = self.make_gui_image_view(parent)
        controls.Add(self.image_view, proportion=0, flag=wx.EXPAND|wx.ALL)
        self.image_view.Bind(EVT_DELETE, self.imageViewDelete)
        self.image_view.Bind(EVT_UPDATE, self.imageViewUpdate)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        return controls

    def make_gui_level(self, parent):
        """Build the control that shows the level.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create objects
        txt = wx.StaticText(parent, wx.ID_ANY, 'Level: ')
        self.map_level = wx.StaticText(parent, wx.ID_ANY, ' ')

        # lay out the controls
        sb = AppStaticBox(parent, 'Map level')
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(txt, border=PackBorder, flag=(wx.ALIGN_CENTER_VERTICAL
                                     |wx.ALIGN_RIGHT|wx.LEFT))
        box.Add(self.map_level, proportion=0, border=PackBorder,
                flag=wx.RIGHT|wx.TOP)

        return box

    def make_gui_mouse(self, parent):
        """Build the mouse part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create objects
        txt = wx.StaticText(parent, wx.ID_ANY, 'Lon/Lat: ')
        self.mouse_position = ROTextCtrl(parent, '', size=(150,-1),
                                         tooltip=('Shows the mouse '
                                                  'longitude and latitude '
                                                  'on the map'))

        # lay out the controls
        sb = AppStaticBox(parent, 'Mouse position')
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(txt, border=PackBorder, flag=(wx.ALIGN_CENTER_VERTICAL
                                     |wx.ALIGN_RIGHT|wx.LEFT))
        box.Add(self.mouse_position, proportion=1, border=PackBorder,
                flag=wx.RIGHT|wx.TOP|wx.BOTTOM)

        return box

    def make_gui_image(self, parent):
        """Build the image part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create widgets
        image_obj = LayerControl(parent, 'Image, map-relative',
                                 filename=DefaultFilename,
                                 placement=DefaultPlacement,
                                 x=DefaultX, y=DefaultY,
                                 offset_x=DefaultOffsetX,
                                 offset_y=DefaultOffsetY)

        return image_obj

    def make_gui_image_view(self, parent):
        """Build the view-relative image part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create widgets
        image_obj = LayerControl(parent, 'Image, view-relative',
                                 filename=DefaultViewFilename,
                                 placement=DefaultViewPlacement,
                                 x=DefaultViewX, y=DefaultViewY,
                                 offset_x=DefaultViewOffsetX,
                                 offset_y=DefaultViewOffsetY)

        return image_obj

    ######
    # event handlers
    ######

##### map-relative image layer

    def imageUpdate(self, event):
        """Display updated image."""

        # remove any previous layer
        if self.image_layer:
            self.pyslip.DeleteLayer(self.image_layer)

        # convert values to sanity for layer attributes
        image = event.filename

        placement = event.placement
        if placement == 'none':
            placement= ''

        x = event.x
        if not x:
            x = 0
        try:
            x = float(x)
        except ValueError:
            x = 0.0

        y = event.y
        if not y:
            y = 0
        try:
            y = float(y)
        except ValueError:
            y = 0.0

        off_x = event.offset_x
        if not off_x:
            off_x = 0
        try:
            off_x = int(off_x)
        except ValueError:
            off_x = 0

        off_y = event.offset_y
        if not off_y:
            off_y = 0
        try:
            off_y = int(off_y)
        except ValueError:
            off_y = 0

        radius = event.radius
        colour = event.colour

        image_data = [(x, y, image, {'placement': placement,
                                     'radius': radius,
                                     'colour': colour,
                                     'offset_x': off_x,
                                     'offset_y': off_y})]
        self.image_layer = self.pyslip.AddImageLayer(image_data, map_rel=True,
                                                     visible=True,
                                                     name='<image_layer>')

    def imageDelete(self, event):
        """Delete the image map-relative layer."""

        if self.image_layer:
            self.pyslip.DeleteLayer(self.image_layer)
        self.image_layer = None

##### view-relative image layer

    def imageViewUpdate(self, event):
        """Display updated image."""

        if self.image_view_layer:
            self.pyslip.DeleteLayer(self.image_view_layer)

        # convert values to sanity for layer attributes
        image = event.filename
        placement = event.placement
        if placement == 'none':
            placement= ''

        x = event.x
        if not x:
            x = 0
        x = int(x)

        y = event.y
        if not y:
            y = 0
        y = int(y)

        off_x = event.offset_x
        if not off_x:
            off_x = 0
        off_x = int(off_x)

        off_y = event.offset_y
        if not off_y:
            off_y = 0
        off_y = int(off_y)

        radius = event.radius
        colour = event.colour

        # create a new image layer
        image_data = [(x, y, image, {'placement': placement,
                                     'radius': radius,
                                     'colour': colour,
                                     'offset_x': off_x,
                                     'offset_y': off_y})]
        self.image_view_layer = self.pyslip.AddImageLayer(image_data,
                                                          map_rel=False,
                                                          visible=True,
                                                          name='<image_layer>')

    def imageViewDelete(self, event):
        """Delete the image view-relative layer."""

        if self.image_view_layer:
            self.pyslip.DeleteLayer(self.image_view_layer)
        self.image_view_layer = None

    def final_setup(self, level, position):
        """Perform final setup.

        level     zoom level required
        position  position to be in centre of view

        We do this in a CallAfter() function for those operations that
        must not be done while the GUI is "fluid".
        """

        self.pyslip.GotoLevelAndPosition(level, position)

    ######
    # Exception handlers
    ######

    def handle_position_event(self, event):
        """Handle a pySlip POSITION event."""

        posn_str = ''
        if event.mposn:
            (lon, lat) = event.mposn
            posn_str = ('%.*f / %.*f' % (LonLatPrecision, lon,
                                         LonLatPrecision, lat))

        self.mouse_position.SetValue(posn_str)

    def handle_level_change(self, event):
        """Handle a pySlip LEVEL event."""

        self.map_level.SetLabel('%d' % event.level)

###############################################################################

if __name__ == '__main__':
    import sys
    import getopt
    import traceback

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        log(msg)
        tkinter_error.tkinter_error(msg)
        sys.exit(1)

    def usage(msg=None):
        if msg:
            print(('*'*80 + '\n%s\n' + '*'*80) % msg)
        print(__doc__)


    # plug our handler into the python system
    sys.excepthook = excepthook

    # decide which tiles to use, default is GMT
    argv = sys.argv[1:]

    try:
        (opts, args) = getopt.getopt(argv, 'dht:', ['debug', 'help', 'tiles='])
    except getopt.error:
        usage()
        sys.exit(1)

    tile_source = 'GMT'
    debug = False
    for (opt, param) in opts:
        if opt in ['-h', '--help']:
            usage()
            sys.exit(0)
        elif opt in ['-d', '--debug']:
            debug = True
        elif opt in ('-t', '--tiles'):
            tile_source = param
    tile_source = tile_source.lower()

    # set up the appropriate tile source
    if tile_source == 'gmt':
        from pyslip.gmt_local_tiles import GMTTiles as Tiles
        tile_dir = 'gmt_tiles'
    elif tile_source == 'osm':
        from pyslip.osm_tiles import OSMTiles as Tiles
        tile_dir = 'osm_tiles'
    else:
        usage('Bad tile source: %s' % tile_source)
        sys.exit(3)


    # start wxPython app
    app = wx.App()
    app_frame = AppFrame(tile_dir=tile_dir) #, levels=[0,1,2,3,4])
    app_frame.Show()

    if debug:
        log('Showing wxpython inspector')
        import wx.lib.inspection
        wx.lib.inspection.InspectionTool().Show()

    app.MainLoop()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test PySlip with multiple instances.

Usage: test_multi_widget.py [-h]
"""


import wx
import pyslip

from pyslip.gmt_local_tiles import GMTTiles
from pyslip.osm_tiles import OSMTiles


######
# Various demo constants
######

DefaultAppSize = (600, 400)
MinW = 400
MinH = 300
MaxW = 1000
MaxH = 800

MinTileLevel = 0
InitViewLevel = 2
InitViewPosition = (100.51, 13.75)      # Bangkok

################################################################################
# The main application frame
################################################################################

class TestFrame(wx.Frame):
    def __init__(self, gmt_tile_dir, osm_tile_dir):
        """Initialize the widget.

        gmt_tile_dir  path to directory of GMT tiles
        osm_tile_dir  directory for OSM tile caching
        """

        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title=('PySlip %s - multiwidget test'
                                 % pyslip.__version__))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        # create the tile source object
        self.gmt_tile_dir = gmt_tile_dir
        self.osm_tile_dir = osm_tile_dir

        # note that we a unique Tile source for each widget
        # sharing directories is OK
        gmt_tile_src_1 = GMTTiles(gmt_tile_dir)
        gmt_tile_src_2 = GMTTiles(gmt_tile_dir)
        osm_tile_src_1 = OSMTiles(osm_tile_dir)
        osm_tile_src_2 = OSMTiles(osm_tile_dir)

        # build the GUI
        box = wx.BoxSizer(wx.VERTICAL)
        gsz = wx.GridSizer(rows=2, cols=2, vgap=5, hgap=5)

        self.pyslip1 = pyslip.PySlip(self.panel, tile_src=gmt_tile_src_1,
                                     min_level=MinTileLevel)
        gsz.Add(self.pyslip1, flag=wx.ALL|wx.EXPAND)

        self.pyslip2 = pyslip.PySlip(self.panel, tile_src=osm_tile_src_1,
                                     min_level=MinTileLevel)
        gsz.Add(self.pyslip2, flag=wx.ALL|wx.EXPAND)

        self.pyslip3 = pyslip.PySlip(self.panel, tile_src=osm_tile_src_2,
                                     min_level=MinTileLevel)
        gsz.Add(self.pyslip3, flag=wx.ALL|wx.EXPAND)

        self.pyslip4 = pyslip.PySlip(self.panel, tile_src=gmt_tile_src_2,
                                     min_level=MinTileLevel)
        gsz.Add(self.pyslip4, flag=wx.ALL|wx.EXPAND)

        box.Add(gsz, proportion=1, flag=wx.ALL|wx.EXPAND)

        self.panel.SetSizer(box)
        self.SetSizeHints(MinW, MinH, MaxW, MaxH)
        self.panel.Fit()
        self.Centre()
        self.Show(True)

        # set initial view position
        self.pyslip1.GotoLevelAndPosition(InitViewLevel, InitViewPosition)
        self.pyslip2.GotoLevelAndPosition(InitViewLevel, InitViewPosition)
        self.pyslip3.GotoLevelAndPosition(InitViewLevel, InitViewPosition)
        self.pyslip4.GotoLevelAndPosition(InitViewLevel, InitViewPosition)

################################################################################

if __name__ == '__main__':
    import sys
    import getopt
    import traceback

    # print some usage information
    def usage(msg=None):
        if msg:
            print(msg+'\n')
        print(__doc__)        # module docstring used

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        print msg
        sys.exit(1)

    # plug our handler into the python system
    sys.excepthook = excepthook

    # decide which tiles to use, default is GMT
    argv = sys.argv[1:]

    try:
        (opts, args) = getopt.getopt(argv, 'h', ['help'])
    except getopt.error:
        usage()
        sys.exit(1)

    for (opt, param) in opts:
        if opt in ['-h', '--help']:
            usage()
            sys.exit(0)

    # set up the tile sources - GMT and OSM
    gmt_tile_dir = 'gmt_tiles'
    osm_tile_dir = 'osm_tiles'

    # start wxPython app
    app = wx.App()
    TestFrame(gmt_tile_dir, osm_tile_dir).Show()
    app.MainLoop()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A _base_ Tiles object for pySlip tiles.

All tile sources should inherit from this base class.
For example, see gmt_local_tiles.py and osm_tiles.py.
"""

import os
import glob
import wx
import pycacheback


######
# Base class for a tile source - handles access to a source of tiles.
######

class Tiles(object):

    DefaultTilesDir = '_=TILES=_'

    """An object to source tiles for pyslip."""

    def __init__(self, tiles, start_level=None, min_level=None, max_level=None):
        """Initialise a Tiles instance.

        tiles        tile cache directory, may contain tiles
        start_level  the level to start on'
        """

        # this should be overridden!
        raise Exception('You must override Tiles.__init__()')

    def SetAvailableCallback(self, callback):
        """Set the "tile now available" callback routine.

        callback  function with signature callback(level, x, y, image, bitmap)

        where 'level' is the level of the tile, 'x' and 'y' are
        the coordinates of the tile and 'image' and 'bitmap' are tile data.
        """

        raise Exception('You need to override Tiles.SetAvailableCallback()')

    def UseLevel(self, level):
        """Prepare to serve tiles from the required level.

        level  The required level

        Returns None if unsuccessful, else something non-None.

        This is dependant on tiles, coordinate system syste, etc, so must
        be fully implemented in child classes.
        """

        # don't forget to cancel any outstanding requests from servers

        raise Exception('You must override Tiles.UseLevel()')

    def GetTile(self, x, y):
        """Get bitmap for tile at tile coords (x, y) and current level.

        x  X coord of tile required (tile coordinates)
        y  Y coord of tile required (tile coordinates)

        Returns bitmap object for the tile image.

        Tile coordinates are measured from map top-left.
        """

        raise Exception('You must override Tiles.GetTile()')

    def GetInfo(self, level):
        """Get tile info for a particular level.

        level  the level to get tile info for

        Returns (num_tiles_x, num_tiles_y, ppd_x, ppd_y).

        Note that ppd_? may be meaningless for some tiles, so its
        value will be None.
        """

        raise Exception('You must override Tiles.GetInfo()')

    def Geo2Tile(self, xgeo, ygeo):
        """Convert geo to tile fractional coordinates for level in use.

        xgeo   geo longitude in degrees
        ygeo   geo latitude in degrees

        Note that we assume the point *is* on the map!
        """

        raise Exception('You must override Tiles.Geo2Tile()')

    def Tile2Geo(self, xtile, ytile):
        """Convert tile fractional coordinates to geo for level in use.

        xtile  tile fractional X coordinate
        ytile  tile fractional Y coordinate

        Note that we assume the point *is* on the map!
        """

        raise Exception('You must override Tiles.Tile2Geo()')



#!/usr/bin/env python
# -*- coding= utf-8 -*-

"""
Test PySlip GototPosition() function.

The idea is to have a set of buttons selecting various geo positions on the OSM
tile map.  When selected, the view would be moved with GotoPosition() and a
map-relative marker would be drawn at that position.  At the same time, a
view-relative marker would be drawn at the centre of the view.  The difference
between the two markers shows errors in the Geo2Tile() & Tile2GEO() functions.
"""


import os
import wx
import pyslip
from pyslip.osm_tiles import OSMTiles as Tiles

# If we have log.py, well and good.  Otherwise ...
try:
    import pyslip.log as log
except ImportError:
    def logit(*args, **kwargs):
        pass
    log = logit
    log.debug = logit
    log.info = logit
    log.warn = logit
    log.error = logit
    log.critical = logit

######
# Various demo constants
######

# demo name/version
DemoVersion = '1.0'
DemoName = "pySlip %s - GotoPosition() test %s" % (pyslip.__version__, DemoVersion)

# initial level and position
InitViewLevel = 3
InitViewPosition = (0.0, 0.0)

# startup size of the application
DefaultAppSize = (800, 665)

# where OSM local tiles live
TileDirectory = 'osm_tiles'

# the number of decimal places in a lon/lat display
LonLatPrecision = 3

# a selection of cities, position from WikiPedia, etc
# format is ((<lon>,<lat>),<name>)
Cities = [((0.0, 51.4778), 'Greenwich, United Kingdom'),
          ((5.33, 60.389444), 'Bergen, Norway'),
          ((151.209444, -33.865), 'Sydney, Australia'),
          ((-77.036667, 38.895111), 'Washington DC, USA'),
          ((132.4547112, 34.3948052), 'Hiroshima (広島市), Japan'),
          ((-8.008889, 31.63), 'Marrakech (مراكش), Morocco'),
          ((18.95, 69.65), 'Tromsø, Norway'),
          ((-70.933333, -53.166667), 'Punta Arenas, Chile'),
          ((168.3475, -46.413056), 'Invercargill, New Zealand'),
          ((-147.723056, 64.843611), 'Fairbanks AK, USA'),
          ((103.851959, 1.290270), 'Singapore'),
          ((-3.2054825, 55.955252), "Maxwell's Birthplace"),
          ((7.6059321, 50.3644457), "Deutsches Eck, Koblenz, Germany"),
         ]

# sizes of various spacers
HSpacerSize = (3,1)         # horizontal in application screen
VSpacerSize = (1,5)         # vertical in control pane

# border width when packing GUI elements
PackBorder = 1


################################################################################
# Override the wx.TextCtrl class to add read-only style and background colour
################################################################################

# background colour for the 'read-only' text field
ControlReadonlyColour = '#ffffcc'

class ROTextCtrl(wx.TextCtrl):
    """Override the wx.TextCtrl widget to get read-only text control which
    has a distinctive background colour."""

    def __init__(self, parent, value, tooltip='', *args, **kwargs):
        wx.TextCtrl.__init__(self, parent, wx.ID_ANY, value=value,
                             style=wx.TE_READONLY, *args, **kwargs)
        self.SetBackgroundColour(ControlReadonlyColour)
        self.SetToolTip(wx.ToolTip(tooltip))

################################################################################
# Override the wx.StaticBox class to show our style
################################################################################

class AppStaticBox(wx.StaticBox):

    def __init__(self, parent, label, *args, **kwargs):
        if label:
            label = '  ' + label + '  '
        wx.StaticBox.__init__(self, parent, wx.ID_ANY, label, *args, **kwargs)

################################################################################
# The main application frame
################################################################################

class AppFrame(wx.Frame):
    def __init__(self, tile_dir=TileDirectory):
        wx.Frame.__init__(self, None, size=DefaultAppSize, title=DemoName)
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        self.tile_directory = tile_dir
        self.tile_source = Tiles(tile_dir)

        # the data objects for map and view layers
        self.map_layer = None
        self.view_layer = None

        # build the GUI
        self.make_gui(self.panel)

        # finally, set up application window position
        self.Centre()

        # bind events to handlers
        self.pyslip.Bind(pyslip.EVT_PYSLIP_POSITION, self.handle_position_event)
        self.pyslip.Bind(pyslip.EVT_PYSLIP_LEVEL, self.handle_level_change)

        # finally, goto desired level and position
        self.pyslip.GotoLevelAndPosition(InitViewLevel, InitViewPosition)

#####
# Build the GUI
#####

    def make_gui(self, parent):
        """Create application GUI."""

        # start application layout
        all_display = wx.BoxSizer(wx.HORIZONTAL)
        parent.SetSizer(all_display)

        # put map view in left of horizontal box
        sl_box = self.make_gui_view(parent)
        all_display.Add(sl_box, proportion=1, border=1, flag=wx.EXPAND)

        # small spacer here - separate view and controls
        all_display.AddSpacer(HSpacerSize)

        # add controls to right of spacer
        controls = self.make_gui_controls(parent)
        all_display.Add(controls, proportion=0, border=1)

        parent.SetSizerAndFit(all_display)
#        parent.Fit()

    def make_gui_view(self, parent):
        """Build the map view widget

        parent  reference to the widget parent

        Returns the static box sizer.
        """

        # create gui objects
        sb = AppStaticBox(parent, '')
        tile_object = Tiles(self.tile_directory)
        self.pyslip = pyslip.PySlip(parent, tile_src=self.tile_source)

        # lay out objects
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(self.pyslip, proportion=1, border=1, flag=wx.EXPAND)

        return box

    def make_gui_controls(self, parent):
        """Build the 'controls' part of the GUI

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # all controls in vertical box sizer
        controls = wx.BoxSizer(wx.VERTICAL)

        # add the map level in use widget
        level = self.make_gui_level(parent)
        controls.Add(level, proportion=0, flag=wx.EXPAND|wx.ALL)

        # vertical spacer
        controls.AddSpacer(VSpacerSize)

        # add the mouse position feedback stuff
        mouse = self.make_gui_mouse(parent)
        controls.Add(mouse, proportion=0, flag=wx.EXPAND|wx.ALL)

        # buttons for each point of interest
        self.buttons = {}
        for (num, city) in enumerate(Cities):
            controls.AddSpacer(VSpacerSize)
            (lonlat, name) = city
            btn = wx.Button(parent, num, name)
            controls.Add(btn, proportion=0, flag=wx.EXPAND|wx.ALL)
            btn.Bind(wx.EVT_BUTTON, self.handle_button)
            self.buttons[num] = city
        return controls

    def make_gui_level(self, parent):
        """Build the control that shows the level.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create objects
        txt = wx.StaticText(parent, wx.ID_ANY, 'Level: ')
        self.map_level = wx.StaticText(parent, wx.ID_ANY, ' ')

        # lay out the controls
        sb = AppStaticBox(parent, 'Map level')
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(txt, border=PackBorder, flag=(wx.ALIGN_CENTER_VERTICAL
                                     |wx.ALIGN_RIGHT|wx.LEFT))
        box.Add(self.map_level, proportion=0, border=PackBorder,
                flag=wx.RIGHT|wx.TOP)

        return box

    def make_gui_mouse(self, parent):
        """Build the mouse part of the controls part of GUI.

        parent  reference to parent

        Returns reference to containing sizer object.
        """

        # create objects
        txt = wx.StaticText(parent, wx.ID_ANY, 'Lon/Lat: ')
        self.mouse_position = ROTextCtrl(parent, '', size=(150,-1),
                                         tooltip=('Shows the mouse '
                                                  'longitude and latitude '
                                                  'on the map'))

        # lay out the controls
        sb = AppStaticBox(parent, 'Mouse position')
        box = wx.StaticBoxSizer(sb, orient=wx.HORIZONTAL)
        box.Add(txt, border=PackBorder, flag=(wx.ALIGN_CENTER_VERTICAL
                                     |wx.ALIGN_RIGHT|wx.LEFT))
        box.Add(self.mouse_position, proportion=1, border=PackBorder,
                flag=wx.RIGHT|wx.TOP|wx.BOTTOM)

        return box

    ######
    # Exception handlers
    ######

    def handle_button(self, event):
        """Handle button event."""

        (posn, name) = self.buttons[event.Id]
        self.pyslip.GotoPosition(posn)

        if self.map_layer:
            self.pyslip.DeleteLayer(self.map_layer)
        map_data = [posn]
        point_colour = '#0000ff40'
        self.map_layer = self.pyslip.AddPointLayer(map_data, map_rel=True,
                                                   placement='cc',
                                                   color=point_colour,
                                                   radius=11,
                                                   visible=True,
                                                   name='map_layer')

        if self.view_layer:
            self.pyslip.DeleteLayer(self.view_layer)
        view_data = [(((0,0),(0,-10),(0,0),(0,10),
            (0,0),(-10,0),(0,0),(10,0)),{'colour':'#ff0000ff'},)]
#        poly_colour = '#ff0000ff'
        self.view_layer = self.pyslip.AddPolygonLayer(view_data, map_rel=False,
                                                      placement='cc',
#                                                      colour=poly_colour,
                                                      closed=False,
                                                      visible=True,
                                                      name='view_layer')

    def handle_position_event(self, event):
        """Handle a pySlip POSITION event."""

        posn_str = ''
        if event.mposn:
            (lon, lat) = event.mposn
            posn_str = ('%.*f / %.*f'
                        % (LonLatPrecision, lon, LonLatPrecision, lat))

        self.mouse_position.SetValue(posn_str)

    def handle_level_change(self, event):
        """Handle a pySlip LEVEL event."""

        self.map_level.SetLabel('%d' % event.level)


################################################################################

if __name__ == '__main__':
    import sys
    import traceback
    import pyslip.tkinter_error as tkinter_error

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        log(msg)
        tkinter_error.tkinter_error(msg)
        sys.exit(1)

    # use user tile directory, if supplied
    tile_dir = None
    if len(sys.argv) > 1:
        tile_dir = sys.argv[1]

    # plug our handler into the python system
    sys.excepthook = excepthook

    # start wxPython app
    app = wx.App()
    if tile_dir:
        app_frame = AppFrame(tile_dir=tile_dir)
    else:
        app_frame = AppFrame()
    app_frame.Show()

##    import wx.lib.inspection
##    wx.lib.inspection.InspectionTool().Show()

    app.MainLoop()




#!/usr/bin/env python

"""
A simple logger.

Simple usage:
    import log
    log('A line in the log at the default level (DEBUG)')
    log('A log line at WARN level', Log.WARN)
    log.info('log line issued at INFO level')

Based on the 'borg' recipe from [http://code.activestate.com/recipes/66531/].

Log levels styled on the Python 'logging' module.

Log output includes the module and line # of the log() call.
"""

import os
import sys
import datetime
import traceback


################################################################################
# A simple (?) logger.
################################################################################

class Log(object):

    __shared_state = {}                # this __dict__ shared by ALL instances

    # the predefined logging levels
    CRITICAL = 50
    ERROR = 40
    WARN = 30
    INFO = 20
    DEBUG = 10
    NOTSET = 0

    _level_num_to_name = {NOTSET: 'NOTSET',
                          DEBUG: 'DEBUG',
                          INFO: 'INFO',
                          WARN: 'WARN',
                          ERROR: 'ERROR',
                          CRITICAL: 'CRITICAL'}

    # default maximum length of filename (enforced)
    DefaultMaxFname = 15


    def __init__(self, logfile=None, level=DEBUG, append=False,
                 max_fname=DefaultMaxFname):
        """Initialise the logging object.

        logfile the path to the log file
        level   logging level - don't log below this level
        append  True if log file is appended to
        """

        # make sure we have same state as all other log objects
        self.__dict__ = Log.__shared_state

        self.max_fname = max_fname

        self.sym_level = 'NOTSET'      # set in call to check_level()
        self.level = self.check_level(level)

        # don't allow logfile to change after initially set
        if not hasattr(self, 'logfile'):
            if logfile is None:
                logfile = '%s.log' % __name__
            try:
                if append:
                    self.logfd = open(logfile, 'a')
                else:
                    self.logfd = open(logfile, 'w')
            except IOError:
                # assume we have readonly filesystem
                basefile = os.path.basename(logfile)
                if sys.platform == 'win32':
                    logfile = os.path.join('C:\\', basefile)
                else:
                    logfile = os.path.join('~', basefile)

            # try to open logfile again
            if append:
                self.logfd = open(logfile, 'a')
            else:
                self.logfd = open(logfile, 'w')

            self.logfile = logfile

            self.debug('='*55)
            self.debug('Log started on %s, log level=%s'
                       % (datetime.datetime.now().ctime(),
                          self._level_num_to_name[level]))
            self.debug('-'*55)

    def check_level(self, level):
        """Check the level value for legality.

        level  a numeric logging level

        If 'level' is invalid, raise Exception.  If valid, return value.
        """

        try:
            level = int(level)
        except ValueError:
            msg = "Logging level invalid: '%s'" % str(level)
            print(msg)
            raise Exception(msg)

        if not 0 <= level <= 50:
            msg = "Logging level invalid: '%s'" % str(level)
            print(msg)
            raise Exception(msg)

        return level

    def set_level(self, level):
        """Set logging level."""

        level = self.check_level(level)

        # convert numeric level to symbolic
        sym = self._level_num_to_name.get(level, None)
        if sym is None:
            # not recognized symbolic but it's legal, so interpret as 'XXXX+2'
            sym_10 = 10 * (level/10)
            sym_rem = level - sym_10
            sym = '%s+%d' % (self._level_num_to_name[sym_10], sym_rem)

        self.level = level
        self.sym_level = sym

        self.critical('Logging level set to %02d (%s)' % (level, sym))

    def __call__(self, msg=None, level=None):
        """Call on the logging object.

        msg    message string to log
        level  level to log 'msg' at (if not given, assume self.level)
        """

        # get level to log at
        if level is None:
            level = self.level

        # are we going to log?
        if level < self.level or self.level <= 0:
            return

        if msg is None:
            msg = ''

        # get time
        to = datetime.datetime.now()
        hr = to.hour
        min = to.minute
        sec = to.second
        msec = to.microsecond

        # caller information - look back for first module != <this module name>
        frames = traceback.extract_stack()
        frames.reverse()
        try:
            (_, mod_name) = __name__.rsplit('.', 1)
        except ValueError:
            mod_name = __name__
        for (fpath, lnum, mname, _) in frames:
            (fname, _) = os.path.basename(fpath).rsplit('.', 1)
            if fname != mod_name:
                break

        # get string for log level
        loglevel = self._level_num_to_name[level]

        fname = fname[:self.max_fname]
        self.logfd.write('%02d:%02d:%02d.%06d|%8s|%*s:%-4d|%s\n'
                         % (hr, min, sec, msec, loglevel, self.max_fname,
                            fname, lnum, msg))
        self.logfd.flush()

    def critical(self, msg):
        """Log a message at CRITICAL level."""

        self(msg, self.CRITICAL)

    def error(self, msg):
        """Log a message at ERROR level."""

        self(msg, self.ERROR)

    def warn(self, msg):
        """Log a message at WARN level."""

        self(msg, self.WARN)

    def info(self, msg):
        """Log a message at INFO level."""

        self(msg, self.INFO)

    def debug(self, msg):
        """Log a message at DEBUG level."""

        self(msg, self.DEBUG)

    def __del__(self):
        self.logfd.close()




#!/usr/bin/env python

"""
Test PySlip assumptions.

We make some assumptions in pySlip about relative speeds
of various operations.  Make sure those assumptions hold.
"""


import os
import time
import unittest


class TestAssumptions(unittest.TestCase):

    def test_copy_list(self):
        """Check 'l_poly = list(poly)' gets us a new list.

        At a few places in pySlip we need a copy of a list, not the original.
        We do this by:
            l_poly = list(poly)
            new_poly = l_poly[:]
        Is the final ...[:] required?
        """

        # try to make copy without [:]
        old_list = [1, 2, 3, 4]
        old_list_id = id(old_list)
        new_list = list(old_list)
        new_list_id = id(new_list)

        # make sure we DO have a copy
        msg = ("'new_list = list(old_list)' DOESN'T give us a copy?\n"
                   "id(old_list)=%d, id(new_list)=%d"
               % (old_list_id, new_list_id))
        self.assertTrue(old_list_id != new_list_id, msg)

    def test_copy(self):
        """Test 'new_list = old_list[:]' does give us a copy.

        At a few places in pySlip we need a copy of a list, not the original.
        We do this by:
            new_poly = l_poly[:]
        """

        # try to make a copy with [:]
        old_list = [1, 2, 3, 4]
        old_list_id = id(old_list)
        new_list = old_list[:]
        new_list_id = id(new_list)

        msg = ("'new_list = old_list[:]' DOESN'T give us a copy?\n"
                   "id(old_list)=%d, id(new_list)=%d"
               % (old_list_id, new_list_id))
        self.assertTrue(old_list_id != new_list_id, msg)

    def test_copy2(self):
        """Check 'list(poly)' is faster than 'poly[:]'.

        At a few places in pySlip we need a copy of a list and we do:
            new_poly = list(poly)
        Is this faster than:
            new_poly = poly[:]
        """

        loops = 100000

        # create the old list
        old_list = [1, 2, 3, 4, 5, 6]

        # time list() approach
        start = time.time()
        for _ in xrange(loops):
            new_list = list(old_list)
        list_delta = time.time() - start

        # time copy approach
        start = time.time()
        for _ in xrange(loops):
            new_list = old_list[:]
        copy_delta = time.time() - start

        msg = ("'old_list[:]' is SLOWER than 'list(old_list)'?\n"
                "old_list[:]=%.1f, list(old_list)=%.1f "
                "(list() is %.2f times faster)"
               % (list_delta, copy_delta,
                   (copy_delta/list_delta)))
        self.assertTrue(list_delta > copy_delta, msg)

    def test_dispatch_slower(self):
        """Test that dispatch is faster than function if/elif/else.

        pySlip used to use code like this:
            x = ...
            y = ...
            test = {'ab': 'x+=1;y-=1',
                    'bc': 'x+=2;y+=3',
                    ...
                   }
            exec test['ab']

        Compare the above with something like:
            def test(x, y, place, x_off, y_off):
                if place == 'ab':
                    x += 1
                            y -= 1
                elif place == 'bc':
                    ...

                return (x, y)

            x = ...
            y = ...
            (x, y) = test(x, y, place, x_off, y_off)

        The function approach (which we use) should be faster.
        """

        LOOPS = 1000000

        # check exec timing
        test = {'cc': 'x+=x_off-w2;  y+=y_off-h2',
                'nw': 'x+=x_off;     y+=y_off',
                'cn': 'x+=x_off-w2;  y+=y_off',
                'ne': 'x+=x_off-w;   y+=y_off',
                'ce': 'x+=x_off-w;   y+=y_off-h2',
                'se': 'x+=x_off-w;   y+=y_off-h',
                'cs': 'x+=x_off-w2;  y+=y_off-h',
                'sw': 'x+=x_off;     y+=y_off-h',
                'cw': 'x+=x_off;     y+=y_off-h2',
                None: '',
                False: '',
                '': ''}
        for key in test:
            test[key] = compile(test[key], 'string', 'exec')

        start = time.time()
        for _ in xrange(LOOPS):
            x = 0
            y = 0
            place = 'nw'
            x_off = 1
            y_off = 3
            w = 100
            w2 = w/2
            h = 100
            h2 = h/2
            exec test[place]
        exec_delta = time.time() - start

        # now for function equivalent
        def test(x, y, place, w, h, x_off, y_off):
            w2 = w/2
            h2 = h/2
            if place == 'cc':
                x+=x_off-w2;  y+=y_off-h2
            elif place == 'nw':
                x+=x_off;     y+=y_off
            elif place == 'cn':
                x+=x_off-w2;  y+=y_off
            elif place == 'ne':
                x+=x_off-w;   y+=y_off
            elif place == 'ce':
                x+=x_off-w;   y+=y_off-h2
            elif place == 'se':
                x+=x_off-w;   y+=y_off-h
            elif place == 'cs':
                x+=x_off-w2;  y+=y_off-h
            elif place == 'sw':
                x+=x_off;     y+=y_off-h
            elif place == 'cw':
                x+=x_off;     y+=y_off-h2
         
            return (x, y)

        start = time.time()
        for _ in xrange(LOOPS):
            x = 0
            y = 0
            place = 'nw'
            x_off = 1
            y_off = 3
            w = 100
            h = 100
            (x, y) = test(x, y, place, w, h, x_off, y_off)
        func_delta = time.time() - start

        msg = ("Function if/else is slower than 'exec dispatch[i]'?\n"
                   "exec=%.2fs, function=%.2fs (exec is %.1f times faster)"
               % (exec_delta, func_delta, func_delta/exec_delta))
        self.assertTrue(exec_delta > func_delta, msg)

    def test_copy_faster(self):
        """Test that a[:] copy is slower than copy.deepcopy(a)."""

        import copy

        loops = 100000

        a = [1,2,3,4,5,6,7,8,9,0]   # fake a Z-order list

        start = time.time()
        for _ in xrange(loops):
            b = copy.deepcopy(a)
        copy_delta = time.time() - start

        start = time.time()
        for _ in xrange(loops):
            b = a[:]
        clone_delta = time.time() - start

        msg = ('copy.deepcopy() is faster than clone[:]?\n'
                   'copy=%.2fs, clone=%.2fs'
               % (copy_delta, clone_delta))
        self.assertTrue(clone_delta < copy_delta, msg)

    def test_tuple_faster(self):
        """Test unpacking tuple is faster than data object attributes."""

        class DataObj(object):
            def __init__(self, *args, **kwargs):
                if len(args) > 0:
                    msg = 'DataObj() must be called with keyword args ONLY!'
                    raise RuntimeError(msg)

                self.__dict__ = kwargs

        tuple_obj = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
        data_obj = DataObj(one=1, two=2, three=3, four=4, five=5,
                           six=6, seven=7, eight=8, nine=9, ten=10)

        loops = 100000

        # time tuple object
        start = time.time()
        for _ in xrange(loops):
            (one, two, three, four, five, six, seven, eight, nine, ten) = tuple_obj
        tuple_delta = time.time() - start

        # time data object
        start = time.time()
        for _ in xrange(loops):
            one = data_obj.one
            two = data_obj.two
            three = data_obj.three
            four = data_obj.four
            five = data_obj.five
            six = data_obj.six
            seven = data_obj.seven
            eight = data_obj.eight
            nine = data_obj.nine
            ten = data_obj.ten
        data_delta = time.time() - start

        msg = ('Data object is faster than tuple?\ndata=%.2fs, tuple=%.2fs'
               % (data_delta, tuple_delta))
        self.assertTrue(tuple_delta < data_delta, msg)

################################################################################

if __name__ == '__main__':
    suite = unittest.makeSuite(TestAssumptions,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test PySlip view-relative points.

Usage: test_viewrel_point.py [-h] [-t (OSM|GMT)]
"""


import wx
import pyslip


######
# Various demo constants
######

DefaultAppSize = (600, 400)

MinTileLevel = 0
InitViewLevel = 2
InitViewPosition = (133.87, -23.7)      # Alice Springs

PointViewDataNW = [( 0, 0), ( 2, 0), ( 4, 0), ( 6, 0), ( 8, 0),
                   (10, 0), ( 0, 2), ( 0, 4), ( 0, 6), ( 0, 8),
                   ( 0,10), ( 2, 2), ( 4, 4), ( 6, 6), ( 8, 8),
                   (10,10), (12,12), (14,14), (16,16), (18,18),
                   (20,20)
                  ]

PointViewDataCN = [(  0,  0), ( -2,  2), ( -4,  4), ( -6,  6),
                   ( -8,  8), (-10, 10), (  2,  2), (  4,  4),
                   (  6,  6), (  8,  8), ( 10, 10), (  0,  2),
                   (  0,  4), (  0,  6), (  0,  8), (  0, 10),
                   (  0, 12), (  0, 14), (  0, 16), (  0, 18),
                   (  0, 20)
                  ]

PointViewDataNE = [(  0,  0), ( -2,  0), ( -4,  0), ( -6,  0),
                   ( -8,  0), (-10,  0), (  0,  2), (  0,  4),
                   (  0,  6), (  0,  8), (  0, 10), ( -2,  2),
                   ( -4,  4), ( -6,  6), ( -8,  8), (-10, 10),
                   (-12, 12), (-14, 14), (-16, 16), (-18, 18),
                   (-20, 20)
                  ]

PointViewDataCE = [(  0,  0), ( -2, -2), ( -4, -4), ( -6, -6),
                   ( -8, -8), (-10,-10), ( -2,  2), ( -4,  4),
                   ( -6,  6), ( -8,  8), (-10, 10), ( -2,  0),
                   ( -4,  0), ( -6,  0), ( -8,  0), (-10,  0),
                   (-12,  0), (-14,  0), (-16,  0), (-18,  0),
                   (-20,  0)
                  ]

PointViewDataSE = [(  0,  0), (  0, -2), (  0, -4), (  0, -6),
                   (  0, -8), (  0,-10), ( -2,  0), ( -4,  0),
                   ( -6,  0), ( -8,  0), (-10,  0), ( -2, -2),
                   ( -4, -4), ( -6, -6), ( -8, -8), (-10,-10),
                   (-12,-12), (-14,-14), (-16,-16), (-18,-18),
                   (-20,-20)
                  ]

PointViewDataCS = [(  0,  0), ( -2, -2), ( -4, -4), ( -6, -6),
                   ( -8, -8), (-10,-10), (  2, -2), (  4, -4),
                   (  6, -6), (  8, -8), ( 10,-10), (  0, -2),
                   (  0, -4), (  0, -6), (  0, -8), (  0,-10),
                   (  0,-12), (  0,-14), (  0,-16), (  0,-18),
                   (  0,-20)
                  ]

PointViewDataSW = [(  0,  0), (  0, -2), (  0, -4), (  0, -6),
                   (  0, -8), (  0,-10), (  2,  0), (  4,  0),
                   (  6,  0), (  8,  0), ( 10,  0), (  2, -2),
                   (  4, -4), (  6, -6), (  8, -8), ( 10,-10),
                   ( 12,-12), ( 14,-14), ( 16,-16), ( 18,-18),
                   ( 20,-20)
                  ]

PointViewDataCW = [(  0,  0), (  2, -2), (  4, -4), (  6, -6),
                   (  8, -8), ( 10,-10), (  2,  2), (  4,  4),
                   (  6,  6), (  8,  8), ( 10, 10), (  2,  0),
                   (  4,  0), (  6,  0), (  8,  0), ( 10,  0),
                   ( 12,  0), ( 14,  0), ( 16,  0), ( 18,  0),
                   ( 20,  0)
                  ]

PointViewDataCC = [(  0,  0), (  2, -2), (  4, -4), (  6, -6),
                   (  8, -8), ( 10,-10),
                   (  0,  0), (  2,  2), (  4,  4), (  6,  6),
                   (  8,  8), ( 10, 10),
                   (  0,  0), ( -2, -2), ( -4, -4), ( -6, -6),
                   ( -8, -8), (-10,-10),
                   (  0,  0), ( -2,  2), ( -4,  4), ( -6,  6),
                   ( -8,  8), (-10, 10),
                  ]

################################################################################
# The main application frame
################################################################################

class TestFrame(wx.Frame):
    def __init__(self, tile_dir):
        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title=('PySlip %s - view-relative point test'
                                 % pyslip.__version__))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        # create the tile source object
        self.tile_src = Tiles(tile_dir)

        # build the GUI
        box = wx.BoxSizer(wx.HORIZONTAL)
        self.panel.SetSizer(box)
        self.pyslip = pyslip.PySlip(self.panel, tile_src=self.tile_src,
                                    min_level=MinTileLevel)
        box.Add(self.pyslip, proportion=1, border=1, flag=wx.EXPAND)
        self.panel.SetSizer(box)
        self.panel.Layout()
        self.Centre()
        self.Show(True)

        # add test point layers
        self.pyslip.AddPointLayer(PointViewDataNW, placement='nw',
                                  map_rel=False, colour='blue', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataCN, placement='cn',
                                  map_rel=False, colour='red', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataNE, placement='ne',
                                  map_rel=False, colour='green', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataCE, placement='ce',
                                  map_rel=False, colour='black', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataSE, placement='se',
                                  map_rel=False, colour='yellow', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataCS, placement='cs',
                                  map_rel=False, colour='gray', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataSW, placement='sw',
                                  map_rel=False, colour='#7f7fff', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataCW, placement='cw',
                                  map_rel=False, colour='#ff7f7f', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        self.pyslip.AddPointLayer(PointViewDataCC, placement='cc',
                                  map_rel=False, colour='#7fff7f', radius=2,
                                  offset_x=0, offset_y=0,
                                  name='<point_map_layer>')

        # set initial view position
        self.pyslip.GotoLevelAndPosition(InitViewLevel, InitViewPosition)

################################################################################

if __name__ == '__main__':
    import sys
    import getopt
    import traceback

    # print some usage information
    def usage(msg=None):
        if msg:
            print(msg+'\n')
        print(__doc__)        # module docstring used

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        print msg
        sys.exit(1)

    # plug our handler into the python system
    sys.excepthook = excepthook

    # decide which tiles to use, default is GMT
    argv = sys.argv[1:]

    try:
        (opts, args) = getopt.getopt(argv, 'ht:', ['help', 'tiles='])
    except getopt.error:
        usage()
        sys.exit(1)

    tile_source = 'GMT'
    for (opt, param) in opts:
        if opt in ['-h', '--help']:
            usage()
            sys.exit(0)
        elif opt in ('-t', '--tiles'):
            tile_source = param
    tile_source = tile_source.lower()

    # set up the appropriate tile source
    if tile_source == 'gmt':
        from pyslip.gmt_local_tiles import GMTTiles as Tiles
        tile_dir = 'gmt_tiles'
    elif tile_source == 'osm':
        from pyslip.osm_tiles import OSMTiles as Tiles
        tile_dir = 'osm_tiles'
    else:
        usage('Bad tile source: %s' % tile_source)
        sys.exit(3)


    # start wxPython app
    app = wx.App()
    TestFrame(tile_dir).Show()
    app.MainLoop()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test PySlip map-relative polygons.

Usage: test_maprel_poly.py [-h] [-t (OSM|GMT)]
"""


import wx
import pyslip


######
# Various demo constants
######

DefaultAppSize = (600, 400)

MinTileLevel = 0
InitViewLevel = 2
InitViewPosition = (152.0, -8.0)

# create polygon data
OpenPoly = ((145,5),(135,5),(135,-5),(145,-5))
ClosedPoly = ((170,5),(160,5),(160,-5),(170,-5))
FilledPoly = ((170,-20),(160,-20),(160,-10),(170,-10))
ClosedFilledPoly = ((145,-20),(135,-20),(135,-10),(145,-10))

PolyMapData = [[OpenPoly, {'width': 2}],
               [ClosedPoly, {'width': 10, 'color': '#00ff0040',
                             'closed': True}],
               [FilledPoly, {'colour': 'blue',
                             'filled': True,
                             'fillcolour': '#00ff0022'}],
               [ClosedFilledPoly, {'colour': 'black',
                                   'closed': True,
                                   'filled': True,
                                   'fillcolour': 'yellow'}]]

TextMapData = [(135, 5, 'open', {'placement': 'ce', 'radius': 0}),
               (170, 5, 'closed', {'placement': 'cw', 'radius': 0}),
               (170, -10, 'open but filled (translucent)',
                   {'placement': 'cw', 'radius': 0}),
               (135, -10, 'closed & filled (solid)',
                   {'placement': 'ce', 'radius': 0}),
              ]


################################################################################
# The main application frame
################################################################################

class TestFrame(wx.Frame):
    def __init__(self, tile_dir):
        wx.Frame.__init__(self, None, size=DefaultAppSize,
                          title=('PySlip %s - map-relative polygon test'
                                 % pyslip.__version__))
        self.SetMinSize(DefaultAppSize)
        self.panel = wx.Panel(self, wx.ID_ANY)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.panel.ClearBackground()

        # create the tile source object
        self.tile_src = Tiles(tile_dir)

        # build the GUI
        box = wx.BoxSizer(wx.HORIZONTAL)
        self.panel.SetSizer(box)
        self.pyslip = pyslip.PySlip(self.panel, tile_src=self.tile_src,
                                    min_level=MinTileLevel)
        box.Add(self.pyslip, proportion=1, border=1, flag=wx.EXPAND)
        self.panel.SetSizerAndFit(box)
        self.panel.Layout()
        self.Centre()
        self.Show(True)

        # set initial view position
        self.pyslip.GotoLevelAndPosition(InitViewLevel, InitViewPosition)

        # add test text layer
        self.poly_layer = self.pyslip.AddPolygonLayer(PolyMapData,
                                                      map_rel=True,
                                                      name='<poly_map_layer>',
                                                      size=DefaultAppSize)
        self.text_layer = self.pyslip.AddTextLayer(TextMapData, map_rel=True,
                                                   name='<text_map_layer>')

################################################################################

if __name__ == '__main__':
    import sys
    import getopt
    import traceback

    # print some usage information
    def usage(msg=None):
        if msg:
            print(msg+'\n')
        print(__doc__)        # module docstring used

    # our own handler for uncaught exceptions
    def excepthook(type, value, tb):
        msg = '\n' + '=' * 80
        msg += '\nUncaught exception:\n'
        msg += ''.join(traceback.format_exception(type, value, tb))
        msg += '=' * 80 + '\n'
        print msg
        sys.exit(1)

    # plug our handler into the python system
    sys.excepthook = excepthook

    # decide which tiles to use, default is GMT
    argv = sys.argv[1:]

    try:
        (opts, args) = getopt.getopt(argv, 'ht:', ['help', 'tiles='])
    except getopt.error:
        usage()
        sys.exit(1)

    tile_source = 'GMT'
    for (opt, param) in opts:
        if opt in ['-h', '--help']:
            usage()
            sys.exit(0)
        elif opt in ('-t', '--tiles'):
            tile_source = param
    tile_source = tile_source.lower()

    # set up the appropriate tile source
    if tile_source == 'gmt':
        from pyslip.gmt_local_tiles import GMTTiles as Tiles
        tile_dir = 'gmt_tiles'
    elif tile_source == 'osm':
        from pyslip.osm_tiles import OSMTiles as Tiles
        tile_dir = 'osm_tiles'
    else:
        usage('Bad tile source: %s' % tile_source)
        sys.exit(3)

    # start wxPython app
    app = wx.App()
    TestFrame(tile_dir=tile_dir).Show()
    app.MainLoop()

